/**
 * EasyBeans
 * Copyright (C) 2006 Bull S.A.S.
 * Contact: easybeans@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: MemoryArchive.java 4675 2009-02-20 15:58:43Z benoitf $
 * --------------------------------------------------------------------------
 */
package org.ow2.util.archive.impl;

import java.net.MalformedURLException;
import java.net.URL;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import org.ow2.util.archive.api.IArchive;

/**
 * Emulate an archive in memory.
 * @author Gael Lalire
 */
public class MemoryArchive extends AbsArchiveImpl implements IArchive {

    /**
     * Map which associate an url to a class name.
     */
    private Map<String, URL> urlMap;

    /**
     * Construct an archive in memory.
     */
    public MemoryArchive() {
        this.urlMap = new HashMap<String, URL>();
    }

    /**
     * Construct an archive in memory.
     * @param classLoader this classloader is use to load class by their name
     * @param classNames the name of classes in archive
     */
    public MemoryArchive(final ClassLoader classLoader, final List<String> classNames) {
        this.urlMap = new HashMap<String, URL>();
        addClassResource(classLoader, classNames);
    }

    /**
     * Close the archive.
     * @return true
     */
    public boolean close() {
        return true;
    }

    /**
     * @return ArchiveInMemory
     */
    public String getName() {
        return "ArchiveInMemory";
    }

    /**
     * @param resourceName the resource name
     * @return the url of a class.
     */
    public URL getResource(final String resourceName) {
        return this.urlMap.get(resourceName);
    }

    /**
     * @return all classes url
     */
    public Iterator<URL> getResources() {
        return this.urlMap.values().iterator();
    }

    /**
     * @param resourceName the resource name
     * @return a singleton iterator with url of the class
     */
    public Iterator<URL> getResources(final String resourceName) {
        LinkedList<URL> linkedList = new LinkedList<URL>();
        linkedList.add(this.urlMap.get(resourceName));
        return linkedList.iterator();
    }

    /**
     * The memory archive has no url.
     * @return null
     */
    public URL getURL() {
        try {
            return new URL("file:///" + System.identityHashCode(this));
        } catch (MalformedURLException e) {
            throw new IllegalArgumentException("Cannot build URL", e);
        }
    }

    /**
     * @param resourceName the resource name
     * @param url the url
     */
    public void addResource(final String resourceName, final URL url) {
        urlMap.put(resourceName, url);
    }

    /**
     * @param classLoader this classloader is use to load class by their name
     * @param classNames the name of classes in archive
     */
    public void addClassResource(final ClassLoader classLoader, final List<String> classNames) {
        for (String className : classNames) {
            String resourceName = className.replace('.', '/').concat(".class");
            this.urlMap.put(resourceName, classLoader.getResource(resourceName));
        }
    }

    /**
     * @return all resources name
     */
    public Iterator<String> getEntries() {
        return urlMap.keySet().iterator();
    }

}
